// Licensed Materials - Property of IBM 
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
package com.ibm.websphere.batch.devframework.datastreams.patterns;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.Properties;

import com.ibm.jzos.ZFile;
import com.ibm.websphere.batch.devframework.configuration.BDSFrameworkConstants;
import com.ibm.websphere.batch.devframework.configuration.BDSFWLogger;
import com.ibm.websphere.batch.devframework.datastreams.patternadapter.BDSFileByteReader;

public class ZFileStreamOrientedByteReader extends BDSFileByteReader {
	public static final String CLASSNAME = ZFileStreamOrientedByteReader.class.getName(); 

	protected String dsname = "";
	protected String ds_parameters = "rt";
	protected ZFile zFile;	
	protected BDSFWLogger logger;


	public void initialize(Properties props) {
		
		logger = new BDSFWLogger(props);
		dsname = getRequiredProperty(props,BDSFrameworkConstants.DSNAME_KEY);
		//if (prop != null)
		//{
		//	this.dsname = prop;
		//}
		
		String prop = props.getProperty(BDSFrameworkConstants.DSParameters_KEY);
		if (prop != null)
		{
			this.ds_parameters = prop;
		}
		super.initialize(props);
	}

	public void open() {
		String methodName = "open";
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		try
		{			
			//zFile = new ZFile("//'" + dsname + "'", "rb,type=record,recfm=" + recfm + ",lrecl=" + lrecl +",blksize=" + blksize);
			zFile = new ZFile("//'" + dsname + "'", ds_parameters);     			
			bufferedByteReader = new BufferedInputStream(zFile.getInputStream()); 	
			
			if (logger.isDebugEnabled())
				logger.debug("zFileStreamOrientedByteReader- bufferedByteReader.available= " + bufferedByteReader.available());
			
			if (logger.isDebugEnabled())
				logger.debug("zFileStreamOrientedByteReader- zfile.getInputStream.available= " + zFile.getInputStream().available());
			
			this.byteReaderImpl.processHeader(bufferedByteReader);
		}
		catch (Throwable t)
		{
			throw new RuntimeException(t);
		}     
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.endMeasurementAndSave(methodName);
	}


  /**
	 * Returns true if there is more data to be read
   *
   * d707776.1: rewrite.  
   *
   * !! NOTE: The only thing different between this method and the parent method 
   *          (BDSFileByteReader._hasNext) is that the bufferedByteReader.available()
   *          check has been omitted here.  That's because the available() method 
   *          always returns 0 for ZFile InputStreams that are based on datasets.
	 */
	public boolean _hasNext() throws Exception {
		String methodName = "hasNext";
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		if (logger.isDebugEnabled())
			logger.debug(CLASSNAME + ":" + methodName);

    // d707776.1: If a prefetched record is available -- meaning it hasn't
    // yet been read via the read() method -- then don't do anything here.  
    // Just return true, informing the caller that the stream has more records.  
    // Otherwise, prefetch the next record.
    if (!prefetchedRecordAvailable)
    {
      // Prefetch the next record.
      record = read();
		  if (record != null)
        prefetchedRecordAvailable = true;
    }
    
		if(isPerformanceMeasurementEnabled)
			perfAnalyzer.startMeasurement(methodName);
		
		return prefetchedRecordAvailable;
	}



}
